@@ -1,7 +1,7 @@
 /*********************************************************************
   Blosc - Blocked Shuffling and Compression Library
 
-  Copyright (c) 2021  The Blosc Development Team <blosc@blosc.org>
+  Copyright (c) 2021  Blosc Development Team <blosc@blosc.org>
   https://blosc.org
   License: BSD 3-Clause (see LICENSE.txt)
 
@@ -13,7 +13,7 @@
   @brief Blosc2 header file.
 
   This file contains Blosc2 public API and the structures needed to use it.
-  @author The Blosc Development Team <blosc@blosc.org>
+  @author Blosc Development Team <blosc@blosc.org>
 **********************************************************************/
 
 #ifndef BLOSC_BLOSC2_H
@@ -82,11 +82,11 @@ extern "C" {
 
 /* Version numbers */
 #define BLOSC2_VERSION_MAJOR    2    /* for major interface/format changes  */
-#define BLOSC2_VERSION_MINOR    12   /* for minor interface/format changes  */
-#define BLOSC2_VERSION_RELEASE  0    /* for tweaks, bug-fixes, or development */
+#define BLOSC2_VERSION_MINOR    19   /* for minor interface/format changes  */
+#define BLOSC2_VERSION_RELEASE  1    /* for tweaks, bug-fixes, or development */
 
-#define BLOSC2_VERSION_STRING   "2.12.0"  /* string version.  Sync with above! */
-#define BLOSC2_VERSION_DATE     "$Date:: 2023-12-28 #$"    /* date version */
+#define BLOSC2_VERSION_STRING   "2.19.1"  /* string version.  Sync with above! */
+#define BLOSC2_VERSION_DATE     "$Date:: 2025-07-17 #$"    /* date version year-month-day */
 
 
 /* The maximum number of dimensions for Blosc2 NDim arrays */
@@ -108,12 +108,12 @@ extern "C" {
     do {                                            \
         if ((pointer) == NULL) {                    \
             BLOSC_TRACE_ERROR("Pointer is null");   \
-            return rc;                              \
+            return (rc);                            \
         }                                           \
     } while (0)
 #define BLOSC_ERROR(rc)                             \
     do {                                            \
-        int rc_ = rc;                               \
+        int rc_ = (rc);                             \
         if (rc_ < BLOSC2_ERROR_SUCCESS) {           \
             char *error_msg = print_error(rc_);     \
             BLOSC_TRACE_ERROR("%s", error_msg);     \
@@ -226,7 +226,7 @@ enum {
   BLOSC2_GLOBAL_REGISTERED_FILTERS_START = 32,
   BLOSC2_GLOBAL_REGISTERED_FILTERS_STOP = 159,
   //!< Blosc-registered filters must be between 32 - 159.
-  BLOSC2_GLOBAL_REGISTERED_FILTERS = 4,
+  BLOSC2_GLOBAL_REGISTERED_FILTERS = 5,
   //!< Number of Blosc-registered filters at the moment.
   BLOSC2_USER_REGISTERED_FILTERS_START = 160,
   BLOSC2_USER_REGISTERED_FILTERS_STOP = 255,
@@ -245,14 +245,22 @@ enum {
  */
 enum {
 #ifndef BLOSC_H
-  BLOSC_NOSHUFFLE = 0,   //!< No shuffle (for compatibility with Blosc1).
-  BLOSC_NOFILTER = 0,    //!< No filter.
-  BLOSC_SHUFFLE = 1,     //!< Byte-wise shuffle.
-  BLOSC_BITSHUFFLE = 2,  //!< Bit-wise shuffle.
+  BLOSC_NOSHUFFLE = 0,
+  //!< No shuffle (for compatibility with Blosc1).
+  BLOSC_NOFILTER = 0,
+  //!< No filter.
+  BLOSC_SHUFFLE = 1,
+  //!< Byte-wise shuffle. `filters_meta` does not have any effect here.
+  BLOSC_BITSHUFFLE = 2,
+  //!< Bit-wise shuffle. `filters_meta` does not have any effect here.
 #endif // BLOSC_H
-  BLOSC_DELTA = 3,       //!< Delta filter.
-  BLOSC_TRUNC_PREC = 4,  //!< Truncate mantissa precision; positive values in cparams.filters_meta will keep bits; negative values will reduce bits.
-  BLOSC_LAST_FILTER = 5, //!< sentinel
+  BLOSC_DELTA = 3,
+  //!< Delta filter. `filters_meta` does not have any effect here.
+  BLOSC_TRUNC_PREC = 4,
+  //!< Truncate mantissa precision.
+  //!< Positive values in `filters_meta` will keep bits; negative values will zero bits.
+  BLOSC_LAST_FILTER = 5,
+  //!< sentinel
   BLOSC_LAST_REGISTERED_FILTER = BLOSC2_GLOBAL_REGISTERED_FILTERS_START + BLOSC2_GLOBAL_REGISTERED_FILTERS - 1,
   //!< Determine the last registered filter. It is used to check if a filter is registered or not.
 };
@@ -283,7 +291,8 @@ enum {
  */
 enum {
   BLOSC2_MAXDICTSIZE = 128 * 1024, //!< maximum size for compression dicts
-  BLOSC2_MAXBLOCKSIZE = 536866816  //!< maximum size for blocks
+  BLOSC2_MAXBLOCKSIZE = 536866816, //!< maximum size for blocks
+  BLOSC2_MAXTYPESIZE = BLOSC2_MAXBLOCKSIZE, //!< maximum size for types
 };
 
 
@@ -421,7 +430,7 @@ enum {
   BLOSC2_NO_SPECIAL = 0x0,       //!< no special value
   BLOSC2_SPECIAL_ZERO = 0x1,     //!< zero special value
   BLOSC2_SPECIAL_NAN = 0x2,      //!< NaN special value
-  BLOSC2_SPECIAL_VALUE = 0x3,    //!< generic special value
+  BLOSC2_SPECIAL_VALUE = 0x3,    //!< repeated special value
   BLOSC2_SPECIAL_UNINIT = 0x4,   //!< non initialized values
   BLOSC2_SPECIAL_LASTID = 0x4,   //!< last valid ID for special value (update this adequately)
   BLOSC2_SPECIAL_MASK = 0x7      //!< special value mask (prev IDs cannot be larger than this)
@@ -479,82 +488,33 @@ enum {
 #define BLOSC_ATTRIBUTE_UNUSED
 #endif
 
+/**
+ * @brief Get the string representation of an error code.
+ *
+ * The returned string is a static string, so it should not be modified or freed.
+ * For unknown error codes, it returns "Unknown error".
+ *
+ * @param error_code The error code to get the string representation for.
+ * @return A pointer to a static string representing the error code.
+ */
+BLOSC_EXPORT const char *blosc2_error_string(int error_code);
+
+/**
+ * @brief Get the string representation of an error code.
+ *
+ * This function is identical to #blosc2_error_string, but with a legacy interface.
+ * The returned string is a static read only string, so it should not be modified or freed,
+ * although it is not const.
+ *
+ * Prefer using #blosc2_error_string instead.
+ *
+ * @param rc The error code to get the string representation for.
+ * @return A pointer to a static string representing the error code.
+ */
 static char *print_error(int rc) BLOSC_ATTRIBUTE_UNUSED;
 static char *print_error(int rc) {
-  switch (rc) {
-    case BLOSC2_ERROR_FAILURE:
-      return (char *) "Generic failure";
-    case BLOSC2_ERROR_STREAM:
-      return (char *) "Bad stream";
-    case BLOSC2_ERROR_DATA:
-      return (char *) "Invalid data";
-    case BLOSC2_ERROR_MEMORY_ALLOC:
-      return (char *) "Memory alloc/realloc failure";
-    case BLOSC2_ERROR_READ_BUFFER:
-      return (char *) "Not enough space to read";
-    case BLOSC2_ERROR_WRITE_BUFFER:
-      return (char *) "Not enough space to write";
-    case BLOSC2_ERROR_CODEC_SUPPORT:
-      return (char *) "Codec not supported";
-    case BLOSC2_ERROR_CODEC_PARAM:
-      return (char *) "Invalid parameter supplied to codec";
-    case BLOSC2_ERROR_CODEC_DICT:
-      return (char *) "Codec dictionary error";
-    case BLOSC2_ERROR_VERSION_SUPPORT:
-      return (char *) "Version not supported";
-    case BLOSC2_ERROR_INVALID_HEADER:
-      return (char *) "Invalid value in header";
-    case BLOSC2_ERROR_INVALID_PARAM:
-      return (char *) "Invalid parameter supplied to function";
-    case BLOSC2_ERROR_FILE_READ:
-      return (char *) "File read failure";
-    case BLOSC2_ERROR_FILE_WRITE:
-      return (char *) "File write failure";
-    case BLOSC2_ERROR_FILE_OPEN:
-      return (char *) "File open failure";
-    case BLOSC2_ERROR_NOT_FOUND:
-      return (char *) "Not found";
-    case BLOSC2_ERROR_RUN_LENGTH:
-      return (char *) "Bad run length encoding";
-    case BLOSC2_ERROR_FILTER_PIPELINE:
-      return (char *) "Filter pipeline error";
-    case BLOSC2_ERROR_CHUNK_INSERT:
-      return (char *) "Chunk insert failure";
-    case BLOSC2_ERROR_CHUNK_APPEND:
-      return (char *) "Chunk append failure";
-    case BLOSC2_ERROR_CHUNK_UPDATE:
-      return (char *) "Chunk update failure";
-    case BLOSC2_ERROR_2GB_LIMIT:
-      return (char *) "Sizes larger than 2gb not supported";
-    case BLOSC2_ERROR_SCHUNK_COPY:
-      return (char *) "Super-chunk copy failure";
-    case BLOSC2_ERROR_FRAME_TYPE:
-      return (char *) "Wrong type for frame";
-    case BLOSC2_ERROR_FILE_TRUNCATE:
-      return (char *) "File truncate failure";
-    case BLOSC2_ERROR_THREAD_CREATE:
-      return (char *) "Thread or thread context creation failure";
-    case BLOSC2_ERROR_POSTFILTER:
-      return (char *) "Postfilter failure";
-    case BLOSC2_ERROR_FRAME_SPECIAL:
-      return (char *) "Special frame failure";
-    case BLOSC2_ERROR_SCHUNK_SPECIAL:
-      return (char *) "Special super-chunk failure";
-    case BLOSC2_ERROR_PLUGIN_IO:
-      return (char *) "IO plugin error";
-    case BLOSC2_ERROR_FILE_REMOVE:
-      return (char *) "Remove file failure";
-    case BLOSC2_ERROR_NULL_POINTER:
-      return (char *) "Pointer is null";
-    case BLOSC2_ERROR_INVALID_INDEX:
-      return (char *) "Invalid index";
-    case BLOSC2_ERROR_METALAYER_NOT_FOUND:
-      return (char *) "Metalayer has not been found";
-    case BLOSC2_ERROR_MAX_BUFSIZE_EXCEEDED:
-      return (char *) "Maximum buffersize exceeded";
-    default:
-      return (char *) "Unknown error";
-  }
+  // TODO: remove this function, use blosc2_error_string only
+  return (char *) blosc2_error_string(rc);
 }
 
 
@@ -588,6 +548,7 @@ BLOSC_EXPORT void blosc2_destroy(void);
  * @remark Compression is memory safe and guaranteed not to write @p dest
  * more than what is specified in @p destsize.
  * There is not a minimum for @p src buffer size @p nbytes.
+ * Equivalent to #blosc2_compress.
  *
  * @warning The @p src buffer and the @p dest buffer can not overlap.
  *
@@ -677,6 +638,7 @@ BLOSC_EXPORT int blosc1_compress(int clevel, int doshuffle, size_t typesize,
  *
  * @remark Decompression is memory safe and guaranteed not to write the @p dest
  * buffer more than what is specified in @p destsize.
+ * Similar to #blosc2_decompress.
  *
  * @remark In case you want to keep under control the number of bytes read from
  * source, you can call #blosc1_cbuffer_sizes first to check whether the
@@ -718,6 +680,8 @@ BLOSC_EXPORT int blosc1_decompress(const void* src, void* dest, size_t destsize)
  * The items are returned in @p dest buffer, which has to have enough
  * space for storing all items.
  *
+ * @remark The function #blosc2_getitem is a more complete and secure version.
+ *
  * @param src The compressed buffer from data will be decompressed.
  * @param start The position of the first item (of @p typesize size) from where data
  * will be retrieved.
@@ -757,11 +721,16 @@ BLOSC_EXPORT int blosc2_getitem(const void* src, int32_t srcsize, int start, int
 typedef void (*blosc_threads_callback)(void *callback_data, void (*dojob)(void *), int numjobs, size_t jobdata_elsize, void *jobdata);
 
 /**
-  Set the threading backend for parallel compression/decompression to use `callback` to execute work
-  instead of using the Blosc-managed threads.   This function is *not* thread-safe and should be called
-  before any other Blosc function: it affects all Blosc contexts.  Passing `NULL` uses the default
-  Blosc threading backend.  The `callback_data` argument is passed through to the callback.
- */
+  * @brief Set the threading backend for parallel compression/decompression to use @p callback to execute work
+  * instead of using the Blosc-managed threads.  The @p callback_data argument is passed through to the callback.
+
+  * @param callback: the callback to use. Passing `NULL` uses the default Blosc threading backend.
+  * @param callback_data: the callback data.
+  *
+  * @warning This function is *not* thread-safe and should be called
+  * before any other Blosc function: it affects all Blosc contexts.
+  * @sa https://github.com/Blosc/c-blosc2/pull/81
+  */
 BLOSC_EXPORT void blosc2_set_threads_callback(blosc_threads_callback callback, void *callback_data);
 
 
@@ -899,6 +868,8 @@ BLOSC_EXPORT int blosc2_free_resources(void);
  * returns the @p blocksize (which is used internally for doing the
  * compression by blocks).
  *
+ * @remark Equivalent to function #blosc2_cbuffer_sizes.
+ *
  * @param cbuffer The buffer of compressed data.
  * @param nbytes The pointer where the number of uncompressed bytes will be put.
  * @param cbytes The pointer where the number of compressed bytes will be put.
@@ -1005,7 +976,8 @@ BLOSC_EXPORT const char* blosc2_cbuffer_complib(const void* cbuffer);
 
 enum {
   BLOSC2_IO_FILESYSTEM = 0,
-  BLOSC_IO_LAST_BLOSC_DEFINED = 1,  // sentinel
+  BLOSC2_IO_FILESYSTEM_MMAP = 1,
+  BLOSC_IO_LAST_BLOSC_DEFINED = 2,  // sentinel
   BLOSC_IO_LAST_REGISTERED = 32,  // sentinel
 };
 
@@ -1017,11 +989,11 @@ enum {
 
 typedef void*   (*blosc2_open_cb)(const char *urlpath, const char *mode, void *params);
 typedef int     (*blosc2_close_cb)(void *stream);
-typedef int64_t (*blosc2_tell_cb)(void *stream);
-typedef int     (*blosc2_seek_cb)(void *stream, int64_t offset, int whence);
-typedef int64_t (*blosc2_write_cb)(const void *ptr, int64_t size, int64_t nitems, void *stream);
-typedef int64_t (*blosc2_read_cb)(void *ptr, int64_t size, int64_t nitems, void *stream);
+typedef int64_t (*blosc2_size_cb)(void *stream);
+typedef int64_t (*blosc2_write_cb)(const void *ptr, int64_t size, int64_t nitems, int64_t position, void *stream);
+typedef int64_t (*blosc2_read_cb)(void **ptr, int64_t size, int64_t nitems, int64_t position, void *stream);
 typedef int     (*blosc2_truncate_cb)(void *stream, int64_t size);
+typedef int     (*blosc2_destroy_cb)(void *params);
 
 
 /*
@@ -1032,20 +1004,23 @@ typedef struct {
   //!< The IO identifier.
   char* name;
   //!< The IO name.
+  bool is_allocation_necessary;
+  //!< If true, the caller needs to allocate data for the read function (ptr argument). If false, the read function
+  //!< takes care of memory allocation and stores the address in the allocated_ptr argument.
   blosc2_open_cb open;
   //!< The IO open callback.
   blosc2_close_cb close;
   //!< The IO close callback.
-  blosc2_tell_cb tell;
-  //!< The IO tell callback.
-  blosc2_seek_cb seek;
-  //!< The IO seek callback.
+  blosc2_size_cb size;
+  //!< The IO size callback.
   blosc2_write_cb write;
   //!< The IO write callback.
   blosc2_read_cb read;
   //!< The IO read callback.
   blosc2_truncate_cb truncate;
   //!< The IO truncate callback.
+  blosc2_destroy_cb destroy;
+  //!< The IO destroy callback (called in the end when finished with the schunk).
 } blosc2_io_cb;
 
 
@@ -1076,6 +1051,13 @@ static const blosc2_io BLOSC2_IO_DEFAULTS = {
  */
 BLOSC_EXPORT int blosc2_register_io_cb(const blosc2_io_cb *io);
 
+/**
+ * @brief Get a user-defined input/output callback in Blosc.
+ *
+ * @param id The id of the callback to get.
+ *
+ * @return A pointer containing the desired callback if success. Else a NULL pointer is returned.
+ */
 BLOSC_EXPORT blosc2_io_cb *blosc2_get_io_cb(uint8_t id);
 
 /*********************************************************************
@@ -1109,7 +1091,7 @@ typedef struct {
  *
  * @return 0 if succeeds. Else a negative code is returned.
  */
-BLOSC_EXPORT int register_tuner_private(blosc2_tuner *tuner);
+BLOSC_EXPORT int blosc2_register_tuner(blosc2_tuner *tuner);
 
 
 /**
@@ -1641,6 +1623,31 @@ typedef struct {
  */
 static const blosc2_storage BLOSC2_STORAGE_DEFAULTS = {false, NULL, NULL, NULL, NULL};
 
+/**
+ * @brief Get default struct for compression params meant for user initialization.
+ */
+BLOSC_EXPORT blosc2_cparams blosc2_get_blosc2_cparams_defaults(void);
+
+/**
+ * @brief Get default struct for decompression params meant for user initialization.
+ */
+BLOSC_EXPORT blosc2_dparams blosc2_get_blosc2_dparams_defaults(void);
+
+/**
+ * @brief Get default struct for #blosc2_storage meant for user initialization.
+ */
+BLOSC_EXPORT blosc2_storage blosc2_get_blosc2_storage_defaults(void);
+
+/**
+ * @brief Get default struct for #blosc2_io meant for user initialization.
+ */
+BLOSC_EXPORT blosc2_io blosc2_get_blosc2_io_defaults(void);
+
+/**
+ * @brief Get default struct for #blosc2_stdio_mmap meant for user initialization.
+ */
+BLOSC_EXPORT blosc2_stdio_mmap blosc2_get_blosc2_stdio_mmap_defaults(void);
+
 typedef struct blosc2_frame_s blosc2_frame;   /* opaque type */
 
 /**
@@ -1719,6 +1726,8 @@ typedef struct blosc2_schunk {
   //<! The ndim (mainly for ZFP usage)
   int64_t *blockshape;
   //<! The blockshape (mainly for ZFP usage)
+  bool view;
+  //<! Whether the schunk is a view or not.
 } blosc2_schunk;
 
 
@@ -1804,6 +1813,19 @@ BLOSC_EXPORT blosc2_schunk* blosc2_schunk_open_offset(const char* urlpath, int64
  */
 BLOSC_EXPORT blosc2_schunk* blosc2_schunk_open_udio(const char* urlpath, const blosc2_io *udio);
 
+/**
+ * @brief Open an existing super-chunk (no copy is made) using a user-defined I/O interface.
+ *
+ * @param urlpath The file name.
+ *
+ * @param offset The frame offset.
+ *
+ * @param udio The user-defined I/O interface.
+ *
+ * @return The new super-chunk.
+ */
+BLOSC_EXPORT blosc2_schunk* blosc2_schunk_open_offset_udio(const char* urlpath, int64_t offset, const blosc2_io *udio);
+
 /* @brief Convert a super-chunk into a contiguous frame buffer.
  *
  * @param schunk The super-chunk to convert.
@@ -1911,7 +1933,7 @@ BLOSC_EXPORT int64_t blosc2_schunk_delete_chunk(blosc2_schunk *schunk, int64_t n
  * @return The number of chunks in super-chunk. If some problem is
  * detected, this number will be negative.
  */
-BLOSC_EXPORT int64_t blosc2_schunk_append_buffer(blosc2_schunk *schunk, void *src, int32_t nbytes);
+BLOSC_EXPORT int64_t blosc2_schunk_append_buffer(blosc2_schunk *schunk, const void *src, int32_t nbytes);
 
 /**
  * @brief Decompress and return the @p nchunk chunk of a super-chunk.
@@ -2065,7 +2087,7 @@ BLOSC_EXPORT int64_t blosc2_schunk_frame_len(blosc2_schunk* schunk);
  * If there is an error, a negative value is returned.
  */
 BLOSC_EXPORT int64_t blosc2_schunk_fill_special(blosc2_schunk* schunk, int64_t nitems,
-                                            int special_value, int32_t chunksize);
+                                                int special_value, int32_t chunksize);
 
 
 /*********************************************************************
@@ -2128,46 +2150,6 @@ BLOSC_EXPORT int blosc2_meta_add(blosc2_schunk *schunk, const char *name, uint8_
 BLOSC_EXPORT int blosc2_meta_update(blosc2_schunk *schunk, const char *name, uint8_t *content,
                                     int32_t content_len);
 
-static inline void swap_store(void *dest, const void *pa, int size) {
-  uint8_t *pa_ = (uint8_t *) pa;
-  uint8_t *pa2_ = (uint8_t*)malloc((size_t) size);
-  int i = 1; /* for big/little endian detection */
-  char *p = (char *) &i;
-
-  if (p[0] == 1) {
-    /* little endian */
-    switch (size) {
-      case 8:
-        pa2_[0] = pa_[7];
-        pa2_[1] = pa_[6];
-        pa2_[2] = pa_[5];
-        pa2_[3] = pa_[4];
-        pa2_[4] = pa_[3];
-        pa2_[5] = pa_[2];
-        pa2_[6] = pa_[1];
-        pa2_[7] = pa_[0];
-        break;
-      case 4:
-        pa2_[0] = pa_[3];
-        pa2_[1] = pa_[2];
-        pa2_[2] = pa_[1];
-        pa2_[3] = pa_[0];
-        break;
-      case 2:
-        pa2_[0] = pa_[1];
-        pa2_[1] = pa_[0];
-        break;
-      case 1:
-        pa2_[0] = pa_[0];
-        break;
-      default:
-        fprintf(stderr, "Unhandled nitems: %d\n", size);
-    }
-  }
-  memcpy(dest, pa2_, size);
-  free(pa2_);
-}
-
 /**
  * @brief Get the content out of a metalayer.
  *
@@ -2312,18 +2294,31 @@ typedef struct timespec blosc_timestamp_t;
 #endif
 
 /*
- * Set a timestamp.
+ * @brief Set a timestamp.
+ *
+ * @param timestamp
+ *
  */
 BLOSC_EXPORT void blosc_set_timestamp(blosc_timestamp_t* timestamp);
 
 /*
- * Return the nanoseconds between 2 timestamps.
+ * @brief Get the nanoseconds between 2 timestamps.
+ *
+ * @param start_time
+ * @param end_time
+ *
+ * @return The nanoseconds between start_time and end_time.
  */
 BLOSC_EXPORT double blosc_elapsed_nsecs(blosc_timestamp_t start_time,
                                         blosc_timestamp_t end_time);
 
 /*
- * Return the seconds between 2 timestamps.
+ * @brief Get the seconds between 2 timestamps.
+ *
+ * @param start_time
+ * @param end_time
+ *
+ * @return The seconds between start_time and end_time.
  */
 BLOSC_EXPORT double blosc_elapsed_secs(blosc_timestamp_t start_time,
                                        blosc_timestamp_t end_time);
@@ -2458,16 +2453,29 @@ BLOSC_EXPORT int blosc2_register_filter(blosc2_filter *filter);
 
 /*
  * @brief Remove a directory and its files.
+ *
+ * @param path The directory to remove.
+ *
+ * @return 0 if succeeds. Else a negative code is returned.
  */
 BLOSC_EXPORT int blosc2_remove_dir(const char *path);
 
 /*
  * @brief Remove a file or a directory given by path.
+ *
+ * @param path The file or directory to remove.
+ *
+ * @return 0 if succeeds. Else a negative code is returned.
  */
 BLOSC_EXPORT int blosc2_remove_urlpath(const char *path);
 
 /*
  * @brief Rename a file or a directory given by old_urlpath to new_path.
+ *
+ * @param old_urlpath The original path to the directory or file.
+ * @param new_path The new path to the directory or file.
+ *
+ * @return 0 if succeeds. Else a negative code is returned.
  */
 BLOSC_EXPORT int blosc2_rename_urlpath(char* old_urlpath, char* new_path);
 
@@ -2495,12 +2503,58 @@ BLOSC_EXPORT void blosc2_multidim_to_unidim(const int64_t *index, int8_t ndim, c
  * @param chunks_idx The pointer to the buffer where the indexes will be written. It is the user responsibility
  * to free the buffer.
  *
- *
  * @return The number of chunks needed to get the slice. If some problem is
  * detected, a negative code is returned instead.
  */
 BLOSC_EXPORT int blosc2_get_slice_nchunks(blosc2_schunk* schunk, int64_t *start, int64_t *stop, int64_t **chunks_idx);
 
+
+/*********************************************************************
+  Private functions, these are here for convenience,
+  and are not meant to be included in public docs
+*********************************************************************/
+
+// Private function needed in b2nd.h for deserializing meta
+static inline void swap_store(void *dest, const void *pa, int size) {
+  uint8_t *pa_ = (uint8_t *) pa;
+  uint8_t *pa2_ = (uint8_t*)malloc((size_t) size);
+  int i = 1; /* for big/little endian detection */
+  char *p = (char *) &i;
+
+  if (p[0] == 1) {
+    /* little endian */
+    switch (size) {
+      case 8:
+        pa2_[0] = pa_[7];
+        pa2_[1] = pa_[6];
+        pa2_[2] = pa_[5];
+        pa2_[3] = pa_[4];
+        pa2_[4] = pa_[3];
+        pa2_[5] = pa_[2];
+        pa2_[6] = pa_[1];
+        pa2_[7] = pa_[0];
+        break;
+      case 4:
+        pa2_[0] = pa_[3];
+        pa2_[1] = pa_[2];
+        pa2_[2] = pa_[1];
+        pa2_[3] = pa_[0];
+        break;
+      case 2:
+        pa2_[0] = pa_[1];
+        pa2_[1] = pa_[0];
+        break;
+      case 1:
+        pa2_[0] = pa_[0];
+        break;
+      default:
+        fprintf(stderr, "Unhandled nitems: %d\n", size);
+    }
+  }
+  memcpy(dest, pa2_, size);
+  free(pa2_);
+}
+
 #ifdef __cplusplus
 }
 #endif
